package com.bocloud.cos.booter.controller;

import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.cos.intf.ServiceBillService;
import com.bocloud.cos.model.LineChartResult;
import com.bocloud.cos.params.BillActionParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * 账单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@RestController @RequestMapping("/v1/bills") @Api(tags = "账单管理", value = "账单管理") public class ServiceBillController {

    private final ServiceBillService serviceBillService;
    private Logger logger = LoggerFactory.getLogger(ServiceBillController.class);

    public ServiceBillController(ServiceBillService serviceBillService) {
        this.serviceBillService = serviceBillService;
    }

    /**
     * 获取账单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "获取账单列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "获取账单列表") @GetMapping
    public BocloudResult<GridBean<ServiceBill>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.list(pager, context);
    }

    /**
     * 字典查询账单列表
     *
     * @param condition
     * @param start
     * @param end
     * @param tenantId
     * @param tenant
     * @param zone
     * @param sn
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "字典查询账单列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "字典查询账单列表") @GetMapping("/condition")
    public BocloudResult listByCondition(@RequestParam(value = Common.CONDITION, required = false) String condition,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.listByCondition(condition, context);
    }

    /**
     * 添加账单
     *
     * @param serviceBill
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "添加账单", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "添加账单") @PostMapping public Result create(
        @RequestBody ServiceBill serviceBill, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.create(serviceBill, context);
    }

    /**
     * 修改账单
     *
     * @param id
     * @param serviceBill
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "修改账单", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "修改账单") @PutMapping("/{id}") public Result modify(
        @PathVariable(value = Common.ID) Long id, @RequestBody ServiceBill serviceBill,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        serviceBill.setId(id);
        return serviceBillService.modify(serviceBill, context);
    }

    /**
     * 资源放入回收站生成账单
     *
     * @param orderItemSn
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "资源放入回收站生成账单", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "资源放入回收站生成账单") @PutMapping public Result recycleBill(
        @RequestBody ServiceBill serviceBill) {
        if (null == serviceBill) {
            return new BocloudResult<>(false, "参数格式错误");
        }
        return serviceBillService.recycleBill(serviceBill.getSn());
    }

    /**
     * 删除账单
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "删除账单", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "删除账单") @DeleteMapping("/{id}") public Result remove(
        @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.remove(id, context);
    }

    /**
     * 导出账单列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated @ApiOperation(tags = {"CMC",
        "CSC"}, value = "导出账单列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET}) public void export(Pager pager,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request,
        HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = serviceBillService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "账单统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            logger.error("export service bill error,", e);
        }
    }

    /**
     * 查看账单详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查看账单详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "查看账单详情") @GetMapping("/{id}")
    public BocloudResult<ServiceBill> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceBillService.detail(id);
    }

    /**
     * 获取账单明细列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "获取账单明细列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "获取账单明细列表") @GetMapping("/items")
    public BocloudResult<GridBean<BillItem>> items(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.items(pager, context);
    }

    /**
     * 添加账单明细
     *
     * @param billItem
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "添加账单明细", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "添加账单明细") @PostMapping("/items")
    public Result createItem(@RequestBody BillItem billItem, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.createItem(billItem, context);
    }

    /**
     * 修改账单明细
     *
     * @param id
     * @param item
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "修改账单明细", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "修改账单明细") @PutMapping("/items/{id}")
    public Result modifyItem(@PathVariable(value = Common.ID) Long id, @RequestBody BillItem item,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        item.setId(id);
        return serviceBillService.modifyItem(item, context);
    }

    /**
     * 删除账单明细
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "删除账单明细", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单明细", operation = "删除账单明细") @DeleteMapping("/items/{id}")
    public Result removeItem(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.removeItem(id, context);
    }

    /**
     * 查看账单明细详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查看账单明细详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单明细", operation = "查看账单明细详情") @GetMapping("/items/{id}")
    public BocloudResult<BillItem> detailItem(@PathVariable(value = Common.ID) Long id) {
        return serviceBillService.detailItem(id);
    }

    /**
     * 支付账单/取消账单支付
     *
     * @param action
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "支付账单/取消账单支付", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "支付订单/取消订单支付") @PatchMapping public Result action(
        @RequestBody BillActionParams action, @Value(Common.REQ_CONTEXT) RequestContext context) {

        switch (action.getAction()) {
            case pay:
                return serviceBillService.pay(action.getSn(), action.getCouponId(), context);
            case cancel:
                return serviceBillService.cancel(action.getSn(), context);
            default:
                return new Result(false, "暂不支持此操作");
        }
    }

    /**
     * 查询消费账单概览
     *
     * @param code
     * @param range
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查询消费账单概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "查询消费账单概览") @GetMapping("/stats")
    public BocloudResult<LineChartResult> stats(@RequestParam(value = Common.CODE) String code,
        @RequestParam(value = "range") String range, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.stats(code, range, context);
    }

    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查询消费账单概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "查询消费账单概览") @GetMapping("/report")
    public BocloudResult report(@RequestParam(value = Common.CODE) String code,
        @RequestParam(value = "vendorType") String vendorType, @RequestParam(value = "range") String range,
        @RequestParam(value = "tenantId") Long tenantId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.report(code, vendorType, range, tenantId, context);
    }

    /**
     * 消费账单概览
     *
     * @param code
     * @param start
     * @param end
     * @param city
     * @param managerId
     * @param tenantId
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查询消费账单报表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @Log(module = BoCloudService.COS, target = "账单", operation = "查询消费账单报表") @GetMapping("/dashboard")
    public BocloudResult dashboard(@RequestParam(value = Common.CODE, required = false) String code,
        @RequestParam(value = "start", required = false) String start,
        @RequestParam(value = "end", required = false) String end,
        @RequestParam(value = "city", required = false) String city,
        @RequestParam(value = "managerId", required = false) Long managerId,
        @RequestParam(value = "tenantId", required = false) Long tenantId) {
        return serviceBillService.dashboard(code, start, end, city, managerId, tenantId);
    }
}
