package com.bocloud.cos.booter.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.poi.poifs.filesystem.DirectoryEntry;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.XMLWorker;
import com.itextpdf.tool.xml.XMLWorkerFontProvider;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import com.itextpdf.tool.xml.exceptions.CssResolverException;
import com.itextpdf.tool.xml.html.CssAppliers;
import com.itextpdf.tool.xml.html.CssAppliersImpl;
import com.itextpdf.tool.xml.html.Tags;
import com.itextpdf.tool.xml.parser.XMLParser;
import com.itextpdf.tool.xml.pipeline.css.CSSResolver;
import com.itextpdf.tool.xml.pipeline.css.CssResolverPipeline;
import com.itextpdf.tool.xml.pipeline.end.PdfWriterPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipelineContext;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import me.chyxion.xls.TableToXls;

/**
 * 记录导出工具类
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年3月16日
 *
 */
public class RecordExporter {

	private static final Logger logger = LoggerFactory.getLogger(RecordExporter.class);
	// initialize operation
	private static Configuration cfg = new Configuration(Configuration.VERSION_2_3_25);

	/**
	 * 初始化 freemarker配置
	 */
	static {
		cfg.setDefaultEncoding("utf-8");
		cfg.setLocale(Locale.SIMPLIFIED_CHINESE);
		cfg.setNumberFormat("#");
		cfg.setDateFormat("yyyy-MM-dd");
		cfg.setTimeFormat("HH:mm:ss");
		cfg.setDateTimeFormat(FreemarkerUtil.DATETIME_PATTERN);
		cfg.setClassForTemplateLoading(RecordExporter.class, "/");
	}

	private List<?> records;

	/**
	 * 
	 * @param records
	 */
	public RecordExporter(List<?> records) {
		this.records = records;
	}

	/**
	 * 初始化导出数据需要的map
	 * 
	 * @return
	 */
	public Map<String, Object> initExportData() {
		Map<String, Object> map = new HashMap<>();
		map.put("dateTimeToLong", new FreemarkerUtil.DateTimeToLongModel());
		map.put("records", records);
		return map;
	}

	/**
	 * html页面转excel，字节数组
	 *
	 * @param html
	 * @return
	 * @throws Exception
	 */
	public byte[] html2Excel(String html) throws Exception {
		byte[] bytes;
		try (ByteArrayOutputStream bos = new ByteArrayOutputStream()) {
			TableToXls.process(html, bos);
			bytes = bos.toByteArray();
		}
		return bytes;
	}

	/**
	 * 根据模板名称和数据生成html
	 *
	 * @param map
	 * @param templateName
	 * @return
	 */
	public String getTemplateString(Map<String, Object> map, String templateName) {
		Template template;
		try {
			template = cfg.getTemplate(templateName); // framemaker.ftl为要装载的模板
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		String str = null;
		try {
			str = FreeMarkerTemplateUtils.processTemplateIntoString(template, map);
		} catch (IOException | TemplateException e) {
			logger.error("处理数据失败", e);
		}
		return str;
	}

	/**
	 * 把html转换成pdf，以字节数组的形式返回pdf文件
	 *
	 * @param html
	 * @return pdf字节数组
	 * @throws IOException
	 * @throws DocumentException
	 * @throws CssResolverException
	 */
	public byte[] html2pdf(String html) throws IOException, DocumentException {
		Document document = new Document(PageSize.A4);
		byte[] bytes;
		XMLWorker worker = null;
		PdfWriter writer = null;
		try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
			writer = PdfWriter.getInstance(document, os);
			document.open();
			XMLWorkerFontProvider fontProvider = new XMLWorkerFontProvider() {
				@Override
				public Font getFont(String fontname, String encoding, float size, int style) {
					return super.getFont(fontname == null ? "Noto Sans CJK SC Regular" : fontname, encoding, size,
							style);
				}
			};
			fontProvider.register("template/NotoSansCJKsc-Regular.otf");
			fontProvider.addFontSubstitute("lowagie", "garamond");
			fontProvider.setUseUnicode(true);
			// 使用我们的字体提供器，并将其设置为unicode字体样式
			CssAppliers cssAppliers = new CssAppliersImpl(fontProvider);
			HtmlPipelineContext htmlContext = new HtmlPipelineContext(cssAppliers);
			htmlContext.setTagFactory(Tags.getHtmlTagProcessorFactory());
			CSSResolver cssResolver = XMLWorkerHelper.getInstance().getDefaultCssResolver(true);
			Pipeline<?> pipeline = new CssResolverPipeline(cssResolver,
					new HtmlPipeline(htmlContext, new PdfWriterPipeline(document, writer)));
			worker = new XMLWorker(pipeline, true);
			XMLParser p = new XMLParser(worker);
			Reader inputStreamReader = null;
			try (InputStream inputStream = new ByteArrayInputStream(html.getBytes("UTF-8"))) {
				inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
				p.parse(inputStreamReader);
			}
			document.close();
			bytes = os.toByteArray();
		} finally {
			try {
				document.close();
			} catch (Exception e) {
				logger.error("关闭文档失败");
			}
			try {
				worker.close();
			} catch (Exception e) {
				logger.error("关闭pdfworker失败", e);
			}
			try {
				writer.close();
			} catch (Exception e) {
				logger.error("关闭pdfwriter失败", e);
			}
		}
		return bytes;
	}

	/**
	 * 把html转换成word，以字节数组的形式返回pdf文件
	 *
	 * @param content
	 * @return
	 * @throws Exception
	 */
	public byte[] html2Word(String content) throws Exception {
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		InputStream is = new ByteArrayInputStream(content.getBytes("UTF-8"));
		byte[] bytes;
		try (POIFSFileSystem fs = new POIFSFileSystem()) {
			// 对应于org.apache.poi.hdf.extractor.WordDocument
			DirectoryEntry directory = fs.getRoot();
			directory.createDocument("WordDocument", is);
			fs.writeFilesystem(outputStream);
			bytes = outputStream.toByteArray();
		} finally {
			outputStream.close();
			is.close();
		}
		return bytes;
	}

}
