package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.BillItem;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.internel.entity.BillVmLabel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 账单明细数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("billItemRepository") public class BillItemRepository extends BasicGenericDao<BillItem, Long> {
    private final BasicDao basicDao;

    public BillItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate,
        final BasicDao basicDao) {
        super(jdbcTemplate, npJdbcTemplate);
        this.basicDao = basicDao;
    }

    public List<BillItem> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long tenantId)
        throws Exception {
        String sql = " select a.* from bill_item a left join service_bill b on b.id = a.bill_id where 1 = 1";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != tenantId && 0L != tenantId) {
            sql += " b.tenant_id = :tenantId";
            paramMap.put(Common.TENANTID, tenantId);
        }
        return this.list(BillItem.class, sql, paramMap);
    }

    public int count(List<Param> params, Long tenantId) throws Exception {
        String sql = "select count(1) from bill_item a left join service_bill b on b.id = a.bill_id where 1 = 1";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != tenantId && 0L != tenantId) {
            sql += " b.tenant_id = :tenantId";
            paramMap.put(Common.TENANTID, tenantId);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) throws Exception {
        String sql = "delete from bill_item where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<BillItem> listByBillId(Long billId) throws Exception {
        String sql = "select * from bill_item a where a.bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.list(BillItem.class, sql, params);
    }

    public Boolean removeByBillId(Long billId) throws Exception {
        String sql = "delete from bill_item where bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.execute(sql, params) > 0;
    }

    public BillItem query(Long id) throws Exception {
        String sql = "select * from bill_item a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<BillItem> list = this.list(BillItem.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<BillItem> getByItemId(Long itemId) throws Exception {
        String sql = "select * from bill_item a where a.item_id = :itemId";
        Map<String, Object> params = MapTools.simpleMap("itemId", itemId);
        return this.list(BillItem.class, sql, params);
    }

    public List<Map<String, Object>> top5(String start, String end, Boolean tenant, Boolean zone) throws Exception {
        StringBuilder builder = new StringBuilder("select sum(a.money) total,c.");
        if (null != tenant && tenant) {
            builder.append("name tenant_name");
        } else if (null != zone && zone) {
            builder.append("city");
        }
        builder.append(
            " from bill_item a left join service_bill b on b.id = a.bill_id left join tenant c on c.id = b.tenant_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>();
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            paramMap.put("end", end);
        }
        builder.append(" group by b.tenant_id order by total desc limit 5");
        return this.list(builder.toString(), paramMap);
    }

    public Double summary(Long billId) throws Exception {
        String sql = "select sum(money) summary from bill_item where bill_id = :billId";
        Map<String, Object> param = MapTools.simpleMap("billId", billId);
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> resultMap = list.get(0);
        Object result = resultMap.get("summary");
        if (null == result) {
            return 0.0;
        }
        BigDecimal decimal = (BigDecimal)result;
        return decimal.doubleValue();
    }

    public List<BillItem> list(String start, String end, Long billId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,d.name service_name from bill_item a left join service_bill b on b.id = a.bill_id left join service_sku c on c.id = a.sku_id left join cloud_service d on d.id = c.service_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != billId && billId != 0L) {
            builder.append("  and bill_id = :billId");
            paramMap.put("billId", billId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            paramMap.put("end", end);
        }
        return this.list(BillItem.class, builder.toString(), paramMap);
    }

    public List<Map<String,Object>> productSumary(String start,String end) throws BoCloudDBException{
        String sql = "select sum(a.money) money, cs.name service_name from bill_item a "
            + " left join service_bill sb on sb.id = a.bill_id "
            + " left join cloud_service cs on cs.code = a.service_code "
            + " where sb.status = 'paid' ";
        Map<String, Object> paramMap = new HashMap<>();
        if (null != start) {
            sql +=" and a.gmt_create >= :start";
            paramMap.put("start", start);
        }
        if (null != end) {
            sql +=" and a.gmt_create <= :end";
            paramMap.put("end", end);
        }
        sql +=" GROUP BY service_name";
        return this.list(sql, paramMap);
    }

    public List<Map<String, Object>> summary(String start, String end, Long tenantId, Long projectId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) money,DATE_FORMAT(a.gmt_create,'%Y-%m-%d') gmt_create,d.name service_name from bill_item a left join service_bill b on b.id = a.bill_id left join service_sku c on c.id = a.sku_id left join cloud_service d on d.id = c.service_id where 1 = 1 and b.status = 'paid' ");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and b.tenant_id = :tenantId");
            paramMap.put("tenantId", tenantId);
        }
        if (null != start) {
            builder.append(" and a.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (null != end) {
            builder.append(" and a.gmt_create <= :end");
            paramMap.put("end", end);
        }
        if (null != projectId && 0L != projectId) {
            builder.append(" and b.project_id = :projectId");
            paramMap.put("projectId", projectId);
        }
        builder.append(" group by service_name,gmt_create");
        return this.list(builder.toString(), paramMap);
    }

    /**
     * @param end
     * @param labelValue
     * @return java.util.List<com.bocloud.internel.entity.BillVmLabel>
     * @Author: lxy
     * @Description: 根据标签查询虚拟机账单金额
     * @Date 2021/5/29 16:06
     * @Param start
     * @throws:
     **/
    public List<BillVmLabel> listByLabelType(String start, String end, String labelValue) {
        Map<String, Object> params = new HashMap<>();

        StringBuilder builder =
            new StringBuilder("SELECT cs.name as vm_name,item.sn as sn,bill.gmt_create,label.type,bill.money as money");
        if (StringUtils.isNotEmpty(labelValue)) {
            builder.append(
                ",JSON_UNQUOTE( JSON_EXTRACT( content, JSON_UNQUOTE( REPLACE ( JSON_SEARCH( content, 'all', '"
                    + labelValue + "', NULL, '$[*].key' ), 'key', 'value' ) ))) AS label_value ");
        }
        builder.append(
            " FROM server_label as label,cloud_server AS cs,order_item AS item,bill_item AS bill WHERE cs.id = label.target AND uuid=item.sn AND bill.item_id=item.id");
        if (StringUtils.isNotEmpty(labelValue)) {
            builder.append(" and JSON_CONTAINS( content -> '$[*].key', '\"" + labelValue + "\"', '$' ) ");
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and bill.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and bill.gmt_create <= :end");
            params.put("end", end);
        }
        return this.basicDao.queryForList(builder.toString(), params, BillVmLabel.class).stream()
            .map(o -> (BillVmLabel)o).collect(Collectors.toList());
    }
}
