package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Document;
import com.bocloud.cos.entity.DocumentCategory;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 使用文档数据访问层
 *
 * @author tyl
 * @since 2021/3/8 14:20
 */

@Repository("documentRepository")
public class DocumentRepository extends JdbcGenericDao<Document, Long> {

    private final BasicDao basicDao;

    public DocumentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, BasicDao basicDao) {
        super(jdbcTemplate, npJdbcTemplate);
        this.basicDao = basicDao;
    }

    public List<Document> list(int page, int rows, Map<String, String> sorter, List<Param> params) throws BoCloudDBException {
        StringBuilder sql = new StringBuilder("select a.*,dc.name category from document a left join document_category dc on a.category_id = dc.id where a.is_deleted = 0");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        for (Param param : params) {
            Map<String, Object> paramsMap = param.getParam();
            if (paramsMap.containsKey("categoryId")) {
                sql.append(" and a.category_id in (:categoryId)");
                paramsMap.remove("categoryId");
            }
        }
        sql = new StringBuilder(QueryBuilder.buildRaw(sql.toString(), new Pager(page, rows, params, sorter), "a"));
        return this.list(Document.class, sql.toString(), paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        StringBuilder sql = new StringBuilder("select a.id,a.name,a.content from document a where a.is_deleted = 0");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        for (Param param : params) {
            Map<String, Object> paramsMap = param.getParam();
            if (paramsMap.containsKey("categoryId")) {
                sql.append(" and a.category_id in (:categoryId) ");
                paramsMap.remove("categoryId");
            }
        }
        sql = new StringBuilder(QueryBuilder.build(sql.toString(), new Pager(1, Integer.MAX_VALUE, params, sorter), "a"));
        return this.list(Document.class, sql.toString(), paramMap).stream().map(document -> new SimpleBean(document.getId(), document.getName(), document.getContent())).collect(Collectors.toList());
    }

    public int count(List<Param> params) throws BoCloudDBException {
        StringBuilder sqlBuilder = new StringBuilder("select count(1) from document a where a.is_deleted = 0");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        ListIterator<Param> listIterator = params.listIterator();
        Param categoryParam = null;
        while (listIterator.hasNext()) {
            Param param = listIterator.next();
            Map<String, Object> paramsMap = param.getParam();
            if (paramsMap.containsKey("categoryId")) {
                categoryParam = param;
                sqlBuilder.append(" and a.category_id in (:categoryId) ");
                listIterator.remove();
            }
        }
        String sql = sqlBuilder.toString();
        sql = QueryBuilder.build(sql, params, null, "a");
        if (categoryParam != null) {
            params.add(categoryParam);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<Document> listByStatus(String status) throws BoCloudDBException {
        String sql = "select a.*,dc.name category from document a left join document_category dc on a.category_id = dc.id where a.is_deleted = 0 and a.status = :status ";
        Map<String, Object> paramMap = MapTools.simpleMap("status", status);
        return this.list(Document.class, sql, paramMap);
    }

    public List<Document> listByCategory(Long categoryId) throws BoCloudDBException {
        String sql = "select a.*,dc.name category from document a left join document_category dc on a.category_id = dc.id where a.is_deleted = 0 and a.category_id = :categoryId ";
        Map<String, Object> paramMap = MapTools.simpleMap("categoryId", categoryId);
        return this.list(Document.class, sql, paramMap);
    }

    public Document detail(Long id) throws BoCloudDBException {
        String sql = "select a.*,dc.name category,u.name as username from document a left join document_category dc on a.category_id = dc.id left join user u on u.id = a.creator_id where a.is_deleted = 0 and a.id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Document> list = this.list(Document.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public Document queryByName(String name) throws BoCloudDBException {
        String sql = "select a.*,dc.name category from document a left join document_category dc on a.category_id = dc.id where a.is_deleted = 0 and a.name = :name ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<Document> list = this.list(Document.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public Boolean remove(Long id, Long userId) throws BoCloudDBException {
        String sql = "update document set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public void save(DocumentCategory category) throws BoCloudDBException {
        this.basicDao.saveEntity(category);
    }

    public void removeCateogry(Long id) throws BoCloudDBException {
        String sql = "update  document_category a set  a.is_deleted = 1 where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        this.basicDao.update(sql, params);
    }

    public void updateCateogry(DocumentCategory category) throws BoCloudDBException {
        this.basicDao.updateEntity(category);
    }

    public List<DocumentCategory> listAllCategory() throws BoCloudDBException {
        String sql = "select  a.* from document_category a where  a.is_deleted = 0";
        return this.basicDao.queryForList(sql, DocumentCategory.class).stream().map(o -> (DocumentCategory) o).collect(Collectors.toList());
    }

    public DocumentCategory queryCategoryByName(String name) throws BoCloudDBException {
        String sql = "select  a.* from document_category a where  a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<DocumentCategory> categories = this.basicDao.queryForList(sql, params, DocumentCategory.class).stream().map(o -> (DocumentCategory) o).collect(Collectors.toList());
        return categories.isEmpty() ? null : categories.get(0);
    }

    public DocumentCategory categoryDetail(Long id) throws BoCloudDBException {
        String sql = "select  a.* from document_category a where  a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<DocumentCategory> categories = this.basicDao.queryForList(sql, params, DocumentCategory.class).stream().map(o -> (DocumentCategory) o).collect(Collectors.toList());
        return categories.isEmpty() ? null : categories.get(0);
    }
}
