package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.Document;
import com.bocloud.cos.entity.DocumentCategory;
import com.bocloud.cos.intf.DocumentService;
import com.bocloud.cos.repository.DocumentRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 使用文档业务层
 *
 * @author tyl
 * @since 2021/3/8 15:04
 */
@Service("documentServiceImpl") public class DocumentServiceImpl implements DocumentService {

    Logger logger = LoggerFactory.getLogger(Document.class);
    private final LockFactory lockFactory;
    private final DocumentRepository documentRepository;
    @Value("${upload.storage.path:/home/cmp}") private String storagePath;
    @Value("${upload.storage.host:http://10.40.20.145:60006}") private String storageHost;

    public DocumentServiceImpl(LockFactory lockFactory, DocumentRepository documentRepository) {
        this.lockFactory = lockFactory;
        this.documentRepository = documentRepository;
    }

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter, Boolean simple,
        String condition) {
        try {
            if (!StringUtils.isEmpty(condition)) {
                JSONObject jsonObject = JSONObject.parseObject(condition);
                if (("listByStatus").equals(jsonObject.getString(Common.CONDITION))) {
                    List<Document> documents = documentRepository.listByStatus(jsonObject.getString("status"));
                    return new BocloudResult(true, documents, "查询成功");
                }
            }
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            sorter = Optional.ofNullable(sorter).orElse(new HashMap<>(16, 0.9F));
            for (Param param : params) {
                Map<String, Object> paramsMap = param.getParam();
                if (paramsMap.containsKey("categoryId")) {
                    Long categoryId = Long.parseLong(paramsMap.get("categoryId").toString());
                    List<Long> categoryIds = getChildrenId(categoryId);
                    categoryIds.add(categoryId);
                    paramsMap.put("categoryId", categoryIds);
                }
            }
            int count = documentRepository.count(params);
            GridBean gridBean;
            if (simple) {
                List<SimpleBean> beans = documentRepository.list(params, sorter);
                gridBean = new GridBean(1, Integer.MAX_VALUE, count, beans);
            } else {
                List<Document> list = documentRepository.list(page, rows, sorter, params);
                list.sort(Comparator.comparing(Document::getPriority));
                gridBean = new GridBean(page, rows, count, list);
            }
            return new BocloudResult(true, gridBean, "查询使用文档列表成功");
        } catch (Exception e) {
            logger.error("list documents list error,", e);
            return BocloudResult.FAILED("查询使用文档列表失败:" + e.getMessage());
        }
    }

    private List<Long> getChildrenId(Long categoryId) {
        List<Long> result = new ArrayList<>();
        Map<Long, List<DocumentCategory>> categoryMap =
            documentRepository.listAllCategory().stream().collect(Collectors.groupingBy(DocumentCategory::getParentId));
        List<DocumentCategory> children = categoryMap.get(categoryId);
        if (children != null) {
            result = children.stream().map(DocumentCategory::getId).collect(Collectors.toList());
            for (DocumentCategory child : children) {
                List<Long> childrenIds = getChildrenId(child.getId());
                result.addAll(childrenIds);
            }
        }
        return result;
    }

    @Override public BocloudResult remove(Long id, RequestContext context) {
        String path = Document.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            documentRepository.remove(id, context.getTarget());
            return new BocloudResult(true, null, "删除成功");
        } catch (Exception e) {
            logger.error("remove document error,", e);
            return BocloudResult.FAILED("删除失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult detail(Long id) {
        try {
            Document document = documentRepository.detail(id);
            return new BocloudResult(true, document, "查询文档详情成功");
        } catch (Exception e) {
            logger.error("get document detail error,", e);
            return BocloudResult.FAILED("查询文档详情失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult create(Document document, RequestContext context) {
        try {
            Document existName = documentRepository.queryByName(document.getName());
            Assert.isNull(existName, "该文档已经存在");
            document.setGmtCreate(new Date());
            document.setCreatorId(context.getTarget());
            document.setDeleted(false);
            document.setStatus(Document.Status.UNPUBLISHED.name());
            documentRepository.save(document);
            return new BocloudResult(true, document, "创建使用文档成功");
        } catch (Exception e) {
            logger.error("create document success,", e);
            return BocloudResult.FAILED("创建使用文档失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult modify(Long id, Document document, RequestContext context) {
        String path = Document.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Document exist = documentRepository.detail(id);
            Assert.notNull(exist, "操作文档不存在");
            if (!exist.getName().equals(document.getName())) {
                Document existName = documentRepository.queryByName(document.getName());
                Assert.isNull(existName, "该文档已经存在");
                exist.setName(document.getName());
            }
            exist.setStatus(document.getStatus());
            exist.setGmtModify(new Date());
            exist.setMenderId(context.getTarget());
            exist.setPriority(document.getPriority());
            exist.setContent(document.getContent());
            documentRepository.update(exist);
            return new BocloudResult(true, exist, "修改成功");
        } catch (Exception e) {
            logger.error("modify document info error,", e);
            return BocloudResult.FAILED("修改失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult categoryTree() {
        try {
            Map<Long, List<DocumentCategory>> categoryMap = documentRepository.listAllCategory().stream()
                .collect(Collectors.groupingBy(DocumentCategory::getParentId));
            List<DocumentCategory> rootDocument = Optional.ofNullable(categoryMap.get(0L)).orElse(new ArrayList<>());
            buildCategoryTree(rootDocument, categoryMap);
            return new BocloudResult(true, rootDocument, "查询分类成功");
        } catch (Exception e) {
            logger.error("get document category tree error,", e);
            return BocloudResult.FAILED("查询分类失败:" + e.getMessage());
        }
    }

    private void buildCategoryTree(List<DocumentCategory> rootDocument, Map<Long, List<DocumentCategory>> categoryMap) {
        for (DocumentCategory category : rootDocument) {
            List<DocumentCategory> children = categoryMap.get(category.getId());
            if (children != null && !children.isEmpty()) {
                buildCategoryTree(children, categoryMap);
                category.setChildren(children);
            } else {
                category.setChildren(new ArrayList<>(0));
            }
        }
    }

    @Override public BocloudResult categoryDetail(Long id) {
        try {
            DocumentCategory category = documentRepository.categoryDetail(id);
            return new BocloudResult(true, category, "查询文档分类详情成功");
        } catch (Exception e) {
            logger.error("get category detail error,", e);
            return BocloudResult.FAILED("查询文档分类详情失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult createCategory(DocumentCategory category) {
        try {
            DocumentCategory existCategory = documentRepository.queryCategoryByName(category.getName());
            Assert.isNull(existCategory, "该分类已经存在");
            category.setDeleted(true);
            documentRepository.save(category);
            return new BocloudResult(true, null, "创建文档分类成功");
        } catch (Exception e) {
            logger.error("create document category error,", e);
            return BocloudResult.FAILED("创建文档分类失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult removeCategory(Long id) {
        String path = DocumentCategory.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<Document> documents = documentRepository.listByCategory(id);
            Assert.isTrue(documents.isEmpty(), "该分类下存在帮助文档，不可删除");
            documentRepository.removeCateogry(id);
            return new BocloudResult(true, null, "删除文档分类成功");
        } catch (Exception e) {
            logger.error("remove document category info error", e);
            return BocloudResult.FAILED("删除文档分类失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult modifyCategory(Long id, DocumentCategory category) {
        String path = DocumentCategory.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            DocumentCategory existCategory = documentRepository.categoryDetail(id);
            Assert.notNull(existCategory, "该分类不存在");
            if (!existCategory.getName().equals(category.getName())) {
                DocumentCategory existName = documentRepository.queryCategoryByName(category.getName());
                Assert.isNull(existName, "该分类已存在");
                existCategory.setName(category.getName());
            }
            documentRepository.updateCateogry(existCategory);
            return new BocloudResult(true, null, "修改文档分类成功");
        } catch (Exception e) {
            logger.error("modify document category info error", e);
            return BocloudResult.FAILED("修改文档分类失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult upload(MultipartFile multipartFile) {
        try {
            String originalFilename = multipartFile.getOriginalFilename();
            org.springframework.util.Assert.notNull(originalFilename, "文件名不能为空");
            String fileName =
                System.currentTimeMillis() + originalFilename.substring(originalFilename.lastIndexOf("."));
            String filePath = "/attachment/docs/files/";
            String path = storagePath + filePath;
            File file = new File(path, fileName);
            if (!file.exists()) {
                file.mkdirs();
            }
            multipartFile.transferTo(file);
            String resultPath = filePath + fileName;
//            String resultPath = storageHost + filePath + fileName;
            return new BocloudResult(true, resultPath, "图片上传成功");
        } catch (Exception e) {
            logger.error("update document picture error,", e);
            return BocloudResult.FAILED("图片上传失败:" + e.getMessage());
        }
    }

    @Override public BocloudResult delete(List<String> paths) {
        try {
            List<String> errorPath = new ArrayList<>();
            for (String path : paths) {
                File file = new File(path.substring(path.indexOf("/docs/files/")));
                if (file.exists()) {
                    if (!file.delete()) {
                        errorPath.add(path);
                    }
                } else {
                    logger.warn("the picture in path [{}] is not exists", path);
                }
            }
            if (errorPath.isEmpty()) {
                return new BocloudResult(true, null, "文档图片删除成功");
            } else {
                return errorPath.size() == paths.size() ? new BocloudResult(false, null, "文档图片删除失败") :
                    new BocloudResult(false, errorPath, "文档图片删除异常");
            }
        } catch (Exception e) {
            logger.error("remove document pictures error error ,", e);
            return BocloudResult.FAILED("文档图片删除失败" + e.getMessage());
        }
    }
}
