/**
 * DynamicReports - Free Java reporting library for creating reports dynamically
 *
 * Copyright (C) 2010 - 2016 Ricardo Mariaca
 * http://www.dynamicreports.org
 *
 * This file is part of DynamicReports.
 *
 * DynamicReports is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DynamicReports is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with DynamicReports. If not, see <http://www.gnu.org/licenses/>.
 */

package com.bocloud.cos.service.jasper.utils;

import net.sf.dynamicreports.report.base.expression.AbstractValueFormatter;
import net.sf.dynamicreports.report.builder.DynamicReports;
import net.sf.dynamicreports.report.builder.ReportTemplateBuilder;
import net.sf.dynamicreports.report.builder.component.ComponentBuilder;
import net.sf.dynamicreports.report.builder.datatype.BigDecimalType;
import net.sf.dynamicreports.report.builder.style.StyleBuilder;
import net.sf.dynamicreports.report.builder.tableofcontents.TableOfContentsCustomizerBuilder;
import net.sf.dynamicreports.report.constant.HorizontalTextAlignment;
import net.sf.dynamicreports.report.constant.VerticalTextAlignment;
import net.sf.dynamicreports.report.definition.ReportParameters;

import java.awt.*;
import java.util.Locale;

import static net.sf.dynamicreports.report.builder.DynamicReports.stl;

/**
 * @author hufy
 */
public class DynamicTemplates {
	public static final StyleBuilder ROOT_STYLE;
	public static final StyleBuilder BOLD_STYLE;
	public static final StyleBuilder ITALIC_STYLE;
	public static final StyleBuilder BOLD_CENTERED_STYLE;
	public static final StyleBuilder BOLD_12_CENTERED_STYLE;
	public static final StyleBuilder BOLD_18_CENTERED_STYLE;
	public static final StyleBuilder BOLD_22_CENTERED_STYLE;
	public static final StyleBuilder COLUMN_STYLE;
	public static final StyleBuilder COLUMN_TITLE_STYLE;
	public static final StyleBuilder GROUP_STYLE;
	public static final StyleBuilder SUBTOTAL_STYLE;

	public static final ReportTemplateBuilder REPORT_TEMPLATE;
	public static final CurrencyType CURRENCY_TYPE;

	static {
		ROOT_STYLE = stl.style().setPadding(2).setFont(stl.font().setFontName("myfont"));
		BOLD_STYLE = DynamicReports.stl.style(ROOT_STYLE).bold();
		ITALIC_STYLE = DynamicReports.stl.style(ROOT_STYLE).italic();
		BOLD_CENTERED_STYLE = DynamicReports.stl.style(BOLD_STYLE).setTextAlignment(HorizontalTextAlignment.CENTER,
				VerticalTextAlignment.MIDDLE);
		BOLD_12_CENTERED_STYLE = DynamicReports.stl.style(BOLD_CENTERED_STYLE).setFontSize(12);
		BOLD_18_CENTERED_STYLE = DynamicReports.stl.style(BOLD_CENTERED_STYLE).setFontSize(18);
		BOLD_22_CENTERED_STYLE = DynamicReports.stl.style(BOLD_CENTERED_STYLE).setFontSize(22);
		COLUMN_STYLE = DynamicReports.stl.style(ROOT_STYLE).setVerticalTextAlignment(VerticalTextAlignment.MIDDLE);
		COLUMN_TITLE_STYLE = DynamicReports.stl.style(COLUMN_STYLE).setBorder(DynamicReports.stl.pen1Point())
				.setHorizontalTextAlignment(HorizontalTextAlignment.CENTER).setBackgroundColor(Color.LIGHT_GRAY).bold();
		GROUP_STYLE = DynamicReports.stl.style(BOLD_STYLE).setHorizontalTextAlignment(HorizontalTextAlignment.LEFT);
		SUBTOTAL_STYLE = DynamicReports.stl.style(BOLD_STYLE).setTopBorder(DynamicReports.stl.pen1Point());

		StyleBuilder crosstabGroupStyle = DynamicReports.stl.style(COLUMN_TITLE_STYLE);
		StyleBuilder crosstabGroupTotalStyle = DynamicReports.stl.style(COLUMN_TITLE_STYLE)
				.setBackgroundColor(new Color(170, 170, 170));
		StyleBuilder crosstabGrandTotalStyle = DynamicReports.stl.style(COLUMN_TITLE_STYLE)
				.setBackgroundColor(new Color(140, 140, 140));
		StyleBuilder crosstabCellStyle = DynamicReports.stl.style(COLUMN_STYLE)
				.setBorder(DynamicReports.stl.pen1Point());

		TableOfContentsCustomizerBuilder tableOfContentsCustomizer = DynamicReports.tableOfContentsCustomizer()
				.setHeadingStyle(0, DynamicReports.stl.style(ROOT_STYLE).bold());

		REPORT_TEMPLATE = DynamicReports.template().setLocale(Locale.ENGLISH).setColumnStyle(COLUMN_STYLE)
				.setColumnTitleStyle(COLUMN_TITLE_STYLE).setGroupStyle(GROUP_STYLE).setGroupTitleStyle(GROUP_STYLE)
				.setSubtotalStyle(SUBTOTAL_STYLE).highlightDetailEvenRows().crosstabHighlightEvenRows()
				.setCrosstabGroupStyle(crosstabGroupStyle).setCrosstabGroupTotalStyle(crosstabGroupTotalStyle)
				.setCrosstabGrandTotalStyle(crosstabGrandTotalStyle).setCrosstabCellStyle(crosstabCellStyle)
				.setTableOfContentsCustomizer(tableOfContentsCustomizer);

		CURRENCY_TYPE = new CurrencyType();
	}

	/**
	 * Creates custom component which is possible to add to any report band
	 * component
	 */
	public static ComponentBuilder<?, ?> createTitleComponent(String label) {
		return DynamicReports.cmp.horizontalList()
				.add(DynamicReports.cmp.text(label).setStyle(BOLD_18_CENTERED_STYLE)
						.setHorizontalTextAlignment(HorizontalTextAlignment.RIGHT))
				.newRow().add(DynamicReports.cmp.line()).newRow().add(DynamicReports.cmp.verticalGap(10));
	}

	public static CurrencyValueFormatter createCurrencyValueFormatter(String label) {
		return new CurrencyValueFormatter(label);
	}

	public static class CurrencyType extends BigDecimalType {
		private static final long serialVersionUID = 1L;

		@Override
		public String getPattern() {
			return "$ #,###.00";
		}
	}

	private static class CurrencyValueFormatter extends AbstractValueFormatter<String, Number> {
		private static final long serialVersionUID = 1L;

		private String label;

		public CurrencyValueFormatter(String label) {
			this.label = label;
		}

		@Override
		public String format(Number value, ReportParameters reportParameters) {
			return label + CURRENCY_TYPE.valueToString(value, reportParameters.getLocale());
		}
	}
}