package com.bocloud.cos.service.resource.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.utils.IdGenerator;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * paas资源操作抽象类
 *
 * @author tyl
 * @since 2021/5/21 9:42
 */
public abstract class AbstractPaasResourceOperate extends AbstractResourceOperate {

    /**
     * 实例id
     */
    String instance;
    Integer extension;
    /**
     * 资源项
     */
    List<ElementModel> elements;
    /**
     * 时长 订购参数
     */
    EmptionModel emptionModel;
    /**
     * 变更操作类型
     */
    AlterationParam.Operation operation;
    /**
     * 服务编码
     */
    final String serviceCode;
    /**
     * 过期时间
     */
    Date expiredTime;
    final Logger logger = LoggerFactory.getLogger(AbstractPaasResourceOperate.class);
    CloudServiceRepository serviceRepository;
    ServiceSkuRepository skuRepository;
    OrderItemRepository itemRepository;
    ServiceOrderRepository orderRepository;
    ServiceCategoryRepository categoryRepository;
    LockFactory lockFactory;
    OrderSnService orderSnService;

    /**
     * 资源申请操作构造
     *
     * @param serviceCode
     * @param category
     * @param emptionModel
     * @param elements
     * @param serviceRepository
     * @param skuRepository
     */
    public AbstractPaasResourceOperate(String serviceCode, ResourceCategory category, EmptionModel emptionModel,
        List<ElementModel> elements, CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        ServiceCategoryRepository categoryRepository, LockFactory lockFactory, OrderSnService orderSnService) {
        super(category);
        this.elements = elements;
        this.emptionModel = emptionModel;
        this.serviceCode = serviceCode;
        this.serviceRepository = serviceRepository;
        this.skuRepository = skuRepository;
        this.categoryRepository = categoryRepository;
        this.lockFactory = lockFactory;
        this.orderSnService = orderSnService;
    }

    /**
     * 资源变更的构造
     *
     * @param serviceCode
     * @param instance
     * @param category
     * @param operation
     * @param elements
     * @param serviceRepository
     * @param skuRepository
     * @param itemRepository
     */
    public AbstractPaasResourceOperate(String serviceCode, String instance, ResourceCategory category,
        AlterationParam.Operation operation, List<ElementModel> elements, Date expiredTime,
        CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        OrderItemRepository itemRepository, LockFactory lockFactory, OrderSnService orderSnService,
        ServiceCategoryRepository categoryRepository) {
        super(category);
        this.instance = instance;
        this.elements = elements;
        this.operation = operation;
        this.serviceCode = serviceCode;
        this.expiredTime = expiredTime;
        this.serviceRepository = serviceRepository;
        this.skuRepository = skuRepository;
        this.itemRepository = itemRepository;
        this.lockFactory = lockFactory;
        this.orderSnService = orderSnService;
        this.categoryRepository = categoryRepository;
    }

    /**
     * 资源延期的构造
     *
     * @param category
     * @param instance
     * @param extension
     * @param serviceCode
     * @param itemRepository
     * @param orderRepository
     */
    public AbstractPaasResourceOperate(String serviceCode, String instance, Integer extension,
        ResourceCategory category, OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
        LockFactory lockFactory, OrderSnService orderSnService) {
        super(category);
        this.instance = instance;
        this.extension = extension;
        this.serviceCode = serviceCode;
        this.itemRepository = itemRepository;
        this.orderRepository = orderRepository;
        this.lockFactory = lockFactory;
        this.orderSnService = orderSnService;
    }

    public AbstractPaasResourceOperate(ResourceCategory category, String serviceCode) {
        super(category);
        this.serviceCode = serviceCode;
    }

    @Override public List<OrderItem> buildApplyItem() {
        List<OrderItem> items = new ArrayList<>();
        try {
            DurationModel duration = emptionModel.getDuration();
            Assert.notEmpty(elements, "订购内容不能为空");
            // 订购数量
            Integer count = emptionModel.getCount();
            Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
            Integer elementCount = 1;
            for (int i = 0; i < count; i++) {
                for (ElementModel element : elements) {
                    String serviceCode = element.getServiceCode();
                    CloudService elementService = serviceRepository.queryByCode(serviceCode);
                    ServiceSku sku = skuRepository.query(element.getSkuId());
                    BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
                    element.setPrice(price);
                    // 构建订单内容 设置服务类型名称
                    if (null != element.getCategoryId() && 0 != element.getCategoryId()) {
                        ServiceCategory category = categoryRepository.query(element.getCategoryId());
                        element.setCategory(category.getName());
                    } else {
                        element.setCategory(elementService.getName());
                    }
                    ServiceCategory serviceCategory = categoryRepository.query(element.getCategoryId());
                    element.setCategory(serviceCategory.getName());
                    JSONObject object = JSONObject.parseObject(serviceCategory.getProps());
                    if (null != object) {
                        elementCount = object.getInteger("nodes");
                    }
                    element.setCount(elementCount);
                    element.setPrice(price);
                    OrderItem orderItem = new OrderItem();
                    orderItem.setCatalog(OrderItem.Catalog.complex);
                    orderItem.setSkuId(element.getSkuId());
                    String itemSn = this.getItemSn(element.getSkuId());
                    Assert.notNull(itemSn, "生成订购单号失败");
                    orderItem.setSn(itemSn);
                    orderItem.setServiceCode(serviceCode);
                    orderItem.setStatus(OrderItemStatus.Opening);
                    orderItem.setDuration(JSONObject.toJSONString(duration));
                    orderItem.setDeleted(false);
                    orderItem.setResources("[]");
                    orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
                    orderItem.setBasicPrice(price.multiply(BigDecimal.valueOf(elementCount)));
                    items.add(orderItem);
                }
            }
        } catch (Exception e) {
            logger.error("build apply item error", e);
        }
        return items;
    }

    @Override public List<OrderItem> buildAlterationItem() {
        Assert.notEmpty(elements, "变更内容不能为空");
        try {
            if (operation == AlterationParam.Operation.Upgrade) {
                ElementModel element = elements.get(0);
                OrderItem upOldItem = itemRepository.queryLastSuccessBySn(instance);
                DurationModel upDuration = JSONObject.parseObject(upOldItem.getDuration(), DurationModel.class);
                //预付费
                if (upDuration.getAmount()!=null && upDuration.getAmount() > 0) {
                    Calendar expiredCalendar = Calendar.getInstance();
                    Calendar effectiveCalendar = Calendar.getInstance();
                    expiredCalendar.setTime(expiredTime);
                    effectiveCalendar.setTime(new Date());
                    if (upDuration.getMode().equals(DurationModel.Mode.Month)) {
                        int expiredYear = expiredCalendar.get(Calendar.YEAR);
                        int effectiveYear = effectiveCalendar.get(Calendar.YEAR);
                        int expired = expiredCalendar.get(Calendar.MONTH);
                        int effective = effectiveCalendar.get(Calendar.MONTH);
                        upDuration.setAmount((expiredYear - effectiveYear) * 12 + (expired - effective));
                    } else if (upDuration.getMode().equals(DurationModel.Mode.Year)) {
                        int expired = expiredCalendar.get(Calendar.YEAR);
                        int effective = effectiveCalendar.get(Calendar.YEAR);
                        upDuration.setAmount(expired - effective);
                    }
                }
                return handlerUpgradeItem(instance, upDuration, element);
            }
            return null;
        } catch (Exception e) {
            logger.error("build alteration item error,", e);
        }
        return null;
    }

    private List<OrderItem> handlerUpgradeItem(String uuid, DurationModel duration, ElementModel element) {
        try {
            Long elementSkuId = element.getSkuId();
            ServiceSku sku = skuRepository.query(elementSkuId);
            CloudService skuService = serviceRepository.query(sku.getServiceId());
            List<SkuSpec> skuSpecs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
            List<OrderItem> result = new ArrayList<>();
            ServiceCategory serviceCategory = categoryRepository.query(sku.getCategoryId());
            JSONObject object = JSONObject.parseObject(serviceCategory.getProps());
            Integer elementCount = 1;
            if (null != object) {
                elementCount = object.getInteger("nodes");
            }
            element.setCount(elementCount);
            element.setCategory(element.getCategory());
            String sn;
            if ("vmware.storage.disk".equals(skuService.getCode())) {
                // 数据盘升级
                Integer dataDisk = skuSpecs.stream().filter(skuSpec -> "disk".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
                List<SpecModel> specModels = new ArrayList<>();
                SpecModel diskSpec = new SpecModel();
                diskSpec.setDisk(dataDisk);
                specModels.add(diskSpec);
                element.setSpecs(specModels);
                OrderItem diskItem = itemRepository.queryDiskSn(instance);
                sn = diskItem.getSn();
            } else {
                Integer cpuTotal = skuSpecs.stream().filter(skuSpec -> "cpu".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
                Integer memTotal = skuSpecs.stream().filter(skuSpec -> "memory".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
                Integer sysDiskTotal = skuSpecs.stream().filter(skuSpec -> "disk".equals(skuSpec.getSpecName()))
                    .map(skuSpec -> Integer.parseInt(skuSpec.getSpecValue())).reduce(Integer::sum).orElse(0);
                List<SpecModel> specModels = new ArrayList<>();
                SpecModel cpuSpec = new SpecModel();
                cpuSpec.setCpu(cpuTotal);
                specModels.add(cpuSpec);
                SpecModel memSpec = new SpecModel();
                memSpec.setMemory(BigDecimal.valueOf(memTotal));
                specModels.add(memSpec);
                SpecModel diskSpec = new SpecModel();
                diskSpec.setDisk(sysDiskTotal);
                specModels.add(diskSpec);
                element.setSpecs(specModels);
                sn = uuid;
            }
            BigDecimal price = OrderPriceUtils.getUnitPrice(skuService, sku, element.getSpecs(), duration);
            element.setPrice(price);
            // 构建变更规格的订单项
            OrderItem upgradeItem = new OrderItem();
            Assert.notNull(sn, "旧订单号查询失败");
            upgradeItem.setSn(sn);
            upgradeItem.setSkuId(elementSkuId);
            upgradeItem.setServiceCode(element.getServiceCode());
            upgradeItem.setStatus(OrderItemStatus.Changing);
            upgradeItem.setDuration(JSONObject.toJSONString(duration));
            upgradeItem.setDeleted(false);
            upgradeItem.setResources("[]");
            upgradeItem.setElements(JSONObject.toJSONString(Lists.newArrayList(elements)));
            upgradeItem.setEffectiveTime(new Date());
            upgradeItem.setExpiredTime(expiredTime);
            upgradeItem.setBasicPrice(price);
            result.add(upgradeItem);
            return result;
        } catch (Exception e) {
            logger.error("build upgrade item error,", e);
            return null;
        }
    }

    public String getItemSn(Long skuId) {
        if (null == skuId || 0L == skuId) {
            // 生成订单号，增加分布式锁
            String itemPath = IdGenerator.class.getSimpleName() + "_order_item_sn";
            try (BocloudACLock lock = lockFactory.getACLock(itemPath)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                return IdGenerator.getInstance().nextId();
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return null;
            }
        } else {
            BocloudResult<String> snResult = orderSnService.generate(skuId);
            Assert.isTrue(snResult.isSuccess(), snResult.getMessage());
            return snResult.getData();
        }
    }
}
